package com.tourism.webadmin.app.website.controller;

import cn.hutool.json.JSONUtil;
import com.tourism.common.core.annotation.DisableDataFilter;
import com.tourism.common.core.object.ResponseResult;
import com.tourism.webadmin.app.website.dto.InteractionMessageDto;
import com.tourism.webadmin.app.website.dto.SystemMessageDTO;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@Tag(name = "系统消息")
@RestController
@DisableDataFilter
@Slf4j
@RequestMapping("/website/systemMessageController")
public class SystemMessageController {

    @Autowired
    private  RabbitTemplate rabbitTemplate;

    /**
     * 系统消息
     * @param params 消息体
     * @return {@link ResponseResult }<{@link Void }>
     */
    @PostMapping("/sendSysMessage")
    public ResponseResult<Void> sendMessage(@RequestBody SystemMessageDTO params) {
        rabbitTemplate.convertAndSend("systemQueue", "systemRouter", JSONUtil.toJsonStr(params));
        return ResponseResult.success();
    }

    /**
     * 互动消息
     * @param params 消息体
     * @return {@link ResponseResult }<{@link Void }>
     */
    @PostMapping("/sendInteractionMessage")
    public ResponseResult<Void> sendInteractionMessage(@RequestBody InteractionMessageDto params) {
        rabbitTemplate.convertAndSend("interactQueue", "interactRouter", JSONUtil.toJsonStr(params));
        return ResponseResult.success();
    }


    /**
     * 关注消息
     * @param params 消息体
     * @return {@link ResponseResult }<{@link Void }>
     */
    @PostMapping("/sendFollowMessage")
    public ResponseResult<Void> sendFollowMessage(@RequestBody SystemMessageDTO params) {
        rabbitTemplate.convertAndSend("focusQueue", "focusRouter", JSONUtil.toJsonStr(params));
        return ResponseResult.success();
    }
}
